var getEntitiesKind = function(kind) {
	if(kind !== null) {
		switch(kind) {
			case 'group':
				return 'GROUP';
			case 'classification':
				return 'ACCOUNT_CATEGORY';
			case 'reportingUpTo':
				return 'REPORTING_UP_TO';
		}
	}
}

var getAttributeText = function(attr, subtype) {
	if(attr !== null && subtype !== null) {
	    switch (attr) {
	        case 'risk':
	        	return 'RISK';
	        case 'loginCount':
	        	return 'LOGIN_COUNT';
	        case 'type':
	        	return 'ACCOUNT_TYPE';
	        case 'cost':
	        	if(subtype === 'ACCOUNT' || subtype === 'ORPHAN') {
	        		return 'ACCOUNT_COST';
	        	} else if(subtype === 'PERMISSION') {
	        		return 'PERMISSION_COST';
	        	} else {
	        		return 'COST';
	        	}
	        case 'jobTitle':
	        	//user case. no space needed.
	        	return 'JOB_TITLE'; //needs to be TITLE}
	        case 'jobCode':
	        	return 'JOB_CODE';
	        case 'department':
	        	//user case. no space needed.
	        	return 'DEPARTMENT';
	        case 'location':
	        	return 'LOCATION';
	        case 'employeeType':
	        	return 'EMPLOYEE_TYPE';
	        case 'lastLogin':
	        	return 'LAST_LOGIN_DATE';
	        case 'lastOrphanReview':
	        	return 'LAST_ORPHAN_REVIEW';
	        case 'owner':
	        	if(subtype === 'APPLICATION') {
	        		return 'PERMISSION_OWNER';
	        	} else if(subtype === 'PERMISSION') {
	        		return 'PERMISSION_OWNER';
	        	} else {
	        		break;
	        	}
	    	case 'owners':
	    		return 'OWNERS';
	        case 'supervisor':
	        	return 'MANAGED_BY';
	        case 'description':
	        	return 'DESCRIPTION';
	        case 'dn':
	        	return 'LDAP_DISTINGUISHED_NAME';
	        case 'tags':
	        	return 'TAGS';
	        case 'name':
	        	if (subtype === 'APPLICATION')
	    			return 'APPLICATION_NAME';
	    		else
	        		return 'NAME';
	        case 'staticPermission':
	        	return 'STATIC_PERMISSION_FLAG';
	        case 'collectedOwner':
	        	return 'PERMISSION_OWNER_MAPPING';
	        case 'createdBy':
	        	return 'CREATED_BY';
	    	case 'changedBy':
	    		return 'CHANGED_BY';
	    }
	}                            
};

var getOperatorText = function(op) {
	if (op !== null) {
	    switch (op) {
	        case 'GT':
	            return 'GREATER_THAN';
	        case 'LT':
	            return 'LESS_THAN';
	        case 'EQ':
	            return 'EQUAL_TO';
	        case 'GE':
	            return 'GREATER_THAN_EQ';
	        case 'LE':
	            return 'LESS_THAN_EQ';
	    }
	}                        
};

var getDaysAgoOperatorText = function(op) {
	if (op !== null) {
		switch (op) {
			case 'GT':
				return 'MORE_THAN';
			case 'LT':
				return 'WITHIN';
		}
	}
}

var getDaysAheadOperatorText = function(op) {
	if (op !== null) {
		switch (op) {
			case 'GT':
				return 'MORE_THAN';
			case 'LT':
				return 'WITHIN';
		}
	}
}

var parseAccountInfo = function(jsonObj) {
	var resultList = [];
	if(jsonObj.selectedAccts !== undefined) {
		//with selected accounts
		resultList.push('SELECT_ACCOUNTS');
		
		//getting the account names
		resultList.push('<b>' + jsonObj.selectedAccts.map(function (selectedAccount) {return selectedAccount.displayName;}).join(' | ') + '</b>');

		if(jsonObj.acctCriteria !== undefined) {
			//with selected accounts with criteria

			if(jsonObj.acctConjunction === 'AllOf') {
				//AND_ACCOUNTS_MATCH_ALL=as well as accounts that match all of the following selection criteria
				resultList.push('AND_ACCOUNTS_MATCH_ALL');
			} 
			else {
				//AND_ACCOUNTS_MATCH_ANY=as well as accounts that match any of the following selection criteria
				resultList.push('AND_ACCOUNTS_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.acctCriteria, 'ACCOUNT'));
		} else {
			resultList.push('<br>');
		}
	} 
	else {
		if(jsonObj.acctCriteria !== undefined) {
			//without selected accounts with criteria
			if(jsonObj.acctConjunction === 'AllOf') {
				//ACCOUNTS_MATCH_ALL=select accounts that match all of the following selection criteria
				resultList.push('ACCOUNTS_MATCH_ALL');
			} 
			else {
				//ACCOUNTS_MATCH_ANY=select accounts that match any of the following selection criteria
				resultList.push('ACCOUNTS_MATCH_ANY');
			}
			
			resultList = resultList.concat(parseCriteria(jsonObj.acctCriteria, 'ACCOUNT'));
		} 
		else {
			//without selected accounts and without criteria
			resultList.push('ALL_ACCOUNTS');
			resultList.push('<br>');
		}
	}
	
	return resultList;
}

var parseApplicationInfo = function(jsonObj) {
	var resultList = [];
	if(jsonObj.selectedApps !== undefined) {
		//with selected accounts
		resultList.push('SELECT_APPLICATIONS');
		
		//getting the account names
		resultList.push('<b>' + jsonObj.selectedApps.map(function (selectedApp) {return selectedApp.displayName;}).join(' | ') + '</b>');

		if(jsonObj.appCriteria !== undefined) {
			//with selected accounts with criteria

			if(jsonObj.appConjunction === 'AllOf') {
				//AND_ACCOUNTS_MATCH_ALL=as well as accounts that match all of the following selection criteria
				resultList.push('AND_APPLICATIONS_MATCH_ALL');
			} 
			else {
				//AND_ACCOUNTS_MATCH_ANY=as well as accounts that match any of the following selection criteria
				resultList.push('AND_APPLICATIONS_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.appCriteria, 'APPLICATION'));
		} else {
			resultList.push('<br>');
		}
	} 
	else {
		if(jsonObj.appCriteria !== undefined) {
			//without selected accounts with criteria
			if(jsonObj.appConjunction === 'AllOf') {
				//ACCOUNTS_MATCH_ALL=select accounts that match all of the following selection criteria
				resultList.push('APPLICATIONS_MATCH_ALL');
			} 
			else {
				//ACCOUNTS_MATCH_ANY=select accounts that match any of the following selection criteria
				resultList.push('APPLICATIONS_MATCH_ANY');
			}
			
			resultList = resultList.concat(parseCriteria(jsonObj.appCriteria, 'APPLICATION'));
		} 
		else {
			//without selected accounts and without criteria
			resultList.push('ALL_APPLICATIONS');
			resultList.push('<br>');
		}
	}
	
	return resultList;
}

var parsePermissionInfo = function (jsonObj) {
	var resultList = [];
	if(jsonObj.selectedPerms !== undefined) {
		//with selected accounts
		resultList.push('SELECT_PERMISSIONS');
		
		//getting the account names
		resultList.push('<b>' + jsonObj.selectedPerms.map(function (selectedPerm) {return selectedPerm.displayName;}).join(' | ') + '</b>');

		if(jsonObj.permCriteria !== undefined) {
			//with selected accounts with criteria

			if(jsonObj.permConjunction === 'AllOf') {
				//AND_ACCOUNTS_MATCH_ALL=as well as accounts that match all of the following selection criteria
				resultList.push('AND_PERMISSIONS_MATCH_ALL');
			} 
			else {
				//AND_ACCOUNTS_MATCH_ANY=as well as accounts that match any of the following selection criteria
				resultList.push('AND_PERMISSIONS_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.permCriteria, 'PERMISSION'));
		} else {
			resultList.push('<br>');
		}
	} 
	else {
		if(jsonObj.permCriteria !== undefined) {
			//without selected accounts with criteria
			if(jsonObj.permConjunction === 'AllOf') {
				//ACCOUNTS_MATCH_ALL=select accounts that match all of the following selection criteria
				resultList.push('PERMISSIONS_MATCH_ALL');
			} 
			else {
				//ACCOUNTS_MATCH_ANY=select accounts that match any of the following selection criteria
				resultList.push('PERMISSIONS_MATCH_ANY');
			}
			
			resultList = resultList.concat(parseCriteria(jsonObj.permCriteria, 'PERMISSION'));
		} 
		else {
			//without selected accounts and without criteria
			resultList.push('ALL_PERMISSIONS');
			resultList.push('<br>');
		}
	}
	
	return resultList;	
}

var parseUserInfo = function (jsonObj) {
	var resultList = [];
	if(jsonObj.selectedUsers !== undefined) {
		//with selected accounts
		resultList.push('SELECT_USERS');
		
		//getting the account names
		resultList.push('<b>' + jsonObj.selectedUsers.map(function (selectedUser) {return selectedUser.displayName;}).join(' | ') + '</b>');
		
		if(jsonObj.userCriteria !== undefined) {
			//with selected accounts with criteria

			if(jsonObj.userConjunction === 'AllOf') {
				//AND_ACCOUNTS_MATCH_ALL=as well as accounts that match all of the following selection criteria
				resultList.push('AND_USERS_MATCH_ALL');
			} 
			else {
				//AND_ACCOUNTS_MATCH_ANY=as well as accounts that match any of the following selection criteria
				resultList.push('AND_USERS_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.userCriteria));
		} else {
			resultList.push('<br>');
		}
	} 
	else {
		if(jsonObj.userCriteria !== undefined) {
			//without selected accounts with criteria
			if(jsonObj.userConjunction === 'AllOf') {
				//ACCOUNTS_MATCH_ALL=select accounts that match all of the following selection criteria
				resultList.push('USERS_MATCH_ALL');
			} 
			else {
				//ACCOUNTS_MATCH_ANY=select accounts that match any of the following selection criteria
				resultList.push('USERS_MATCH_ANY');
			}
			
			resultList = resultList.concat(parseCriteria(jsonObj.userCriteria));
		} 
		else {
			//without selected accounts and without criteria
			resultList.push('ALL_USERS');
			resultList.push('<br>');
		}
	}
	
	return resultList;	
}

var parseOrphanInfo = function (jsonObj) {
	var resultList = [];

	if(jsonObj.selectedOrphanAccts !== undefined) {
		//with selected orphans
		resultList.push('SELECT_ORPHANS');
		
		//getting the orphan names
		resultList.push('<b>' + jsonObj.selectedOrphanAccts.map(function (selectedOrphan) {return selectedOrphan.displayName;}).join(' | ') + '</b>');
		
		if(jsonObj.orphanAcctCriteria !== undefined) {
			//with selected accounts with criteria

			if(jsonObj.orphanAcctConjunction === 'AllOf') {
				//AND_ACCOUNTS_MATCH_ALL=as well as accounts that match all of the following selection criteria
				resultList.push('AND_ORPHANS_MATCH_ALL');
			} 
			else {
				//AND_ACCOUNTS_MATCH_ANY=as well as accounts that match any of the following selection criteria
				resultList.push('AND_ORPHANS_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.orphanAcctCriteria, 'ORPHAN'));
		} else {
			resultList.push('<br>');
		}
	}
	else{
		if(jsonObj.orphanAcctCriteria !== undefined) {
			if(jsonObj.orphanAcctConjunction === 'AllOf'){

				resultList.push('ORPHANS_MATCH_ALL');
			} 
			else {

				resultList.push('ORPHANS_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.orphanAcctCriteria, 'ORPHAN'));
		}

		else {
			resultList.push('ALL_ORPHANS');
			resultList.push('<br>');
		}
	}
	return resultList;
}

var parseRoleInfo = function (jsonObj) {
	var resultList = [];

	if(jsonObj.selectedRoles !== undefined) {
		//with selected roles
		resultList.push('SELECT_ROLES');
		
		//getting the role names
		resultList.push('<b>' + jsonObj.selectedRoles.map(function (selectedRole) {return selectedRole.displayName;}).join(' | ') + '</b>');
		
		if(jsonObj.roleCriteria !== undefined) {

			if(jsonObj.roleConjunction === 'AllOf') {
				resultList.push('AND_ROLES_MATCH_ALL');
			} 
			else {
				resultList.push('AND_ROLES_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.roleCriteria, 'ROLE'));
		} else {
			resultList.push('<br>');
		}
	}
	else{
		if(jsonObj.roleCriteria !== undefined) {
			if(jsonObj.roleConjunction === 'AllOf'){
				resultList.push('ROLES_MATCH_ALL');
			} 
			else {
				resultList.push('ROLES_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.roleCriteria, 'ROLE'));
		}

		else {
			resultList.push('ALL_ROLES');
			resultList.push('<br>');
		}
	}
	return resultList;
}

var parseBRoleInfo = function(jsonObj) {
	var resultList = [];

	if(jsonObj.selectedBRoles !== undefined) {
		resultList.push('SELECT_BROLES');

		resultList.push('<b>' + jsonObj.selectedBRoles.map(function (selectedBrole) {return selectedBrole.displayName;}).join(' | ') + '</b>');

		if(jsonObj.bRoleCriteria !== undefined) {

			if(jsonObj.BroleConjunction === 'AllOf') {
				resultList.push('AND_BROLES_MATCH_ALL');
			} 
			else {
				resultList.push('AND_BROLES_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.bRoleCriteria, 'BROLE'));
		} else {
			resultList.push('<br>');
		}
	}
	else{
		if(jsonObj.bRoleCriteria !== undefined) {
			if(jsonObj.bRoleConjunction === 'AllOf'){
				resultList.push('BROLES_MATCH_ALL');
			} 
			else {
				resultList.push('BROLES_MATCH_ANY');
			}

			resultList = resultList.concat(parseCriteria(jsonObj.bRoleCriteria, 'BROLE'));
		}

		else {
			resultList.push('ALL_BROLES');
			resultList.push('<br>');
		}
	}
	return resultList;
}

var parseCriteria = function (criteriaList, subtype) {
	var resultList = [];
	resultList.push('<br>');
	criteriaList.forEach (
		function(criteria){
			
			if(criteria.attributeKey !== undefined) {
				resultList.push('&nbsp;&nbsp;&nbsp;&nbsp;&#8226');
				resultList.push(getAttributeText(criteria.attributeKey, subtype));
			}

			if(criteria.entities !== undefined) {
				if(criteria.kind !== undefined) {
					resultList.push('&nbsp;&nbsp;&nbsp;&nbsp;&#8226');
					resultList.push(getEntitiesKind(criteria.kind));
				}
				resultList.push('<b>' + criteria.entities.map(function (entity) {return entity.displayName;}).join(' | ') + '</b>');
				resultList.push('<br>');
			} else if(criteria.kind !== undefined) {
				switch(criteria.kind) {
					case 'ncompare':
						resultList.push(getOperatorText(criteria.operator));
						resultList.push('<b>' + criteria.nvalue.toString() + '</b>');
						resultList.push('<br>');
						break;
					case 'agocompare':
						resultList.push(getDaysAgoOperatorText(criteria.operator));
						resultList.push('<b>' + criteria.daysAgo.toString() + '</b>');
						resultList.push('DAYS_AGO');
						resultList.push('<br>');
						break;
					case 'aheadcompare':
						resultList.push(getDaysAheadOperatorText(criteria.operator));
						resultList.push('<b>' + criteria.daysAhead.toString() + '</b>');
						resultList.push('DAYS_FROM_NOW');
						resultList.push('<br>');
						break;
					case 'bcompare':
						resultList.push('<b>' + criteria.bvalue.toString() + '</b>');
						resultList.push('<br>');
						break;
				}
			} else if(criteria.values !== undefined) {
				resultList.push('<b>' + criteria.values.join(' | ') + '</b>');
				resultList.push('<br>');
			}
		}
	)
	resultList.push('<br>');
	return resultList;
}

var localizeJSON = function(jsonStr, localizer) {
	var jsonObj = JSON.parse(jsonStr);
	var resultList = [];
	if(jsonObj !== null && localizer !== null && jsonObj.reviewSubtype !== undefined){
		var subtypes = jsonObj.reviewSubtype;

		subtypes.forEach(
			function(subtype) {
				switch(subtype){
					case 'PERMISSION':
						resultList = resultList.concat(parsePermissionInfo(jsonObj));
						break;

					case 'USER':
						resultList = resultList.concat(parseUserInfo(jsonObj));
						break;

					case 'ACCOUNT':
						resultList = resultList.concat(parseAccountInfo(jsonObj));
						break;

					case 'ORPHAN':
						resultList = resultList.concat(parseOrphanInfo(jsonObj));
						break;

					case 'APPLICATION':
						resultList = resultList.concat(parseApplicationInfo(jsonObj));
						break;

					case 'ROLE':
						resultList = resultList.concat(parseRoleInfo(jsonObj));
						break;
					case 'BUSINESS_ROLE':
						resultList = resultList.concat(parseBRoleInfo(jsonObj));
				}
		})

		return resultList.map(function(str) {
				return localizer.getString(str);
		}).join(' ');
	}	
}